#region License
/* Copyright 2004, 2013 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Chatter.Plugins;
using Chatter.Plugins.Client;
using Chatter.Plugins.Events;
using Chatter.Plugins.Server;

namespace RandomSayingsPlugin
{
    [Plugin("Random sayings")]
	public class RandomSayings: IPlugin, IPluginMenu
	{
		public class SayingsOptions
		{
			public List<string> Sayings;
		}

        IChatController chatController;
		IChatEvents     chatEvents;
		ISettings	    chatSettings;
		SayingsOptions  myOptions;

		public void StartPlugin(IPluginServer server)
		{
			// Cast the chat object to its various forms.
            chatEvents = server.GetInterface<IChatEvents>();
            chatController = server.GetInterface<IChatController>();
			chatSettings = server.GetInterface<ISettings>();

			// Load plugin settings.
			myOptions = (SayingsOptions)chatSettings.Load(typeof(SayingsOptions));

			// Did the settings file fail to load?
			if (myOptions == null)
			{
				myOptions = new SayingsOptions();
				myOptions.Sayings = new List<string>();
			}

			// Start intercepting chat messages.
			chatEvents.Message += OnMessage;
		}

		public void StopPlugin()
		{
			// Stop intercepting chat messages.
			chatEvents.Message -= OnMessage;

			// Save settings.
			chatSettings.Save(myOptions);
		}

		public void ShowMenu(object topMenu)
		{
            var menu = topMenu as MenuItem;
            if (menu == null) { return; }
			menu.MenuItems.Add("&Say something witty", new EventHandler(OnWittySayingMenu));
		}

		string GetWittySaying()
		{
			return myOptions.Sayings.Count > 0 ?(string)myOptions.Sayings
				[new Random().Next(myOptions.Sayings.Count - 1)] : null;
		}

		string GetFormattedWittySaying()
		{
			string Saying = GetWittySaying();

            return Saying != null ? "It is said, \"" + Saying + "\"" : "I have no words of wisdom to give you. Type !saying help for more information.";
		}

		void OnWittySayingMenu(object o, EventArgs e)
		{
			string channel = chatController.ActiveChannel;

			if (channel != null)
			{
				chatController.Message(channel, MessageType.Message,
					GetFormattedWittySaying());
			}
		}

		void OnMessage(object o, MessageEventArgs e)
		{
			if (e.IsChannelMessage && e.MessageType == MessageType.Message && 0 == (e.MessageFlags & MessageFlags.Plugin))
			{
                if (e.Message.StartsWith("!saying help", StringComparison.InvariantCultureIgnoreCase))
                {
                    chatController.Message(e.Recipient, MessageType.Message, "Commands:");
                    chatController.Message(e.Recipient, MessageType.Message, "  !saying               Says something.");
                    chatController.Message(e.Recipient, MessageType.Message, "  !saying add: message  Adds a new saying.");
                    chatController.Message(e.Recipient, MessageType.Message, "  !saying find: message Finds a stored saying.");
                }
				else if (e.Message.StartsWith("!saying add:", StringComparison.InvariantCultureIgnoreCase))
				{
					string Saying = e.Message.Substring("!saying add:".Length).Trim();

					if (Saying.Length > 0)
					{
						myOptions.Sayings.Add(Saying);

						chatController.Message(e.Recipient, MessageType.Action,
							"gains new wisdom, \"" + Saying + "\"");
					}
				}
				else if (e.Message.StartsWith("!saying find:", StringComparison.InvariantCultureIgnoreCase))
				{
                    string SearchTerm = e.Message.Substring("!saying find:".Length).Trim().ToLowerInvariant();

					string BestSaying = "I cannot recall anything similar to that.";
					int    BestWeight = 5;

					foreach(string Saying in myOptions.Sayings)
					{
                        int Weight = StringUtility.StringDistance(SearchTerm, Saying.ToLowerInvariant());

						if (Weight > BestWeight)
						{
							BestSaying = "Indeed. It was once said, \"" + Saying + "\"";
							BestWeight = Weight;
						}
					}

					chatController.Message(e.Recipient, MessageType.Message, BestSaying);
				}
				else if (e.Message.Trim().Equals("!saying", StringComparison.InvariantCultureIgnoreCase))
				{
					string Saying = GetFormattedWittySaying();

					chatController.Message(e.Recipient, MessageType.Message, Saying);
				}
			}
		}
	}
}